#include "stdafx.h"
#include <cstdio>
#include <string>
#include <sstream>
#include <iostream>
#include <iterator>
#include <fstream>
using namespace std;
#include "ImageFileFormat.h"
#include "Image.h"

#include "DataStructures.h"
#include "Controller.h"
extern DebugLogFile g_debug_logfile;

/* returns a CString representation of an int */
CString itos(int i)
{
  stringstream ss;
  ss << i;
  return ss.str().c_str();
}

/* returns a CString representation of the tube, window, and session for debugging purposes */
CString GetTWSString(const ImageInfo& ii)
{
  CString s;
  s.Format("Tube: %s\nWindow: %d\nSession: %d",ii.tube,ii.window,ii.session);
  return s;
}

/* Constructor - initializes dialog to Bartz format */
ImageFileFormat::ImageFileFormat()
{
  SetToBartz();
}

// Seems like this function could be simplified.  Instead of calling SetLengths() to find the
// length of each element in the filename, the function could use wildcards in CFileFind to find
// the actual name of the file.  This would also help, perhaps, by getting rid of the need to know
// the actual experiment name, etc.  
//    An alternate way to make the function cost would be to call SetLengths() on a local 
// temporary copy of the file format info. -- STB 5/7/07
CString ImageFileFormat::GetFileName(const CString& dir, const CString& tube, int window, int session) const
{ 
  // if valid inputs
  if(tube.GetLength() > 0 && window > 0 && session > 0)
  {
    // determine lengths of the elements
    // (Yes, I know, casting away costness is a bad idea, but this class really needs to be cleaned up;
    //  perhaps we should have a SetDirectory function that reinitializes all the lengths -- STB 4/29/09)
    const_cast<ImageFileFormat*>(this)->SetLengths(dir);
    
    CString file_format = "";  ///< used to search for the file
    CString file_name = "";    ///< the filename to return
    CString tmp_str = "";      ///< a temp string object
    OrderElement tmp;          ///< a temp order element
    
    // create file format string
    for(int i = 0; i < NUM_OF_ELEMENTS; i++)
    {
      tmp = m_image_file_format_info.InfoOrder[i];
      if(tmp.included)
      {
        // add prefix first
        file_format += tmp.prefix;

        // add elements depending on type
        if(tmp.element == WINDOW)
        {
          tmp_str.Format("%0" + itos(tmp.length) + "d",window);
          file_format += tmp_str;
        }
        else if(tmp.element == SESSION)        
        {
          tmp_str.Format("%0" + itos(tmp.length) + "d",session);
          file_format += tmp_str;
        }
        else if(tmp.element == TUBE)
        {
          tmp_str.Format("%s",tube);
          file_format += tmp_str;
        }
        else // use wildcard (*) for other elements
        {
          file_format += "*";
        }

        // add element delimiter
        file_format += m_image_file_format_info.delim;
      } // if included
    } // for number of elements in filename
    
    // replace last delimiter with a period, then add proper file extension
    file_format.SetAt(file_format.GetLength()-1,'.');
    file_format += m_image_file_format_info.filetype;
    
    CString searchstr;           ///< search string

    // add path information
    searchstr = dir+file_format;

    // search for files
    CFileFind ff;              ///< used to find files
    if(ff.FindFile(searchstr))
    {
      ff.FindNextFile();
      file_name = dir+ff.GetFileName();
    } // if file found

    return file_name;
  }

  return "";
}

CString ImageFileFormat::GetFileSearchString() const
{
  CString file_format = "";
  OrderElement tmp;

  for(int i = 0; i < NUM_OF_ELEMENTS; i++)
  {
    tmp = m_image_file_format_info.InfoOrder[i];
    if(tmp.included)
    {
      file_format += tmp.prefix;
      file_format += "*";
      file_format += m_image_file_format_info.delim;
    } // if included
  } // for number of elements in filename

  file_format.SetAt(file_format.GetLength()-1,'.');
  file_format += m_image_file_format_info.filetype;

  return file_format;
}

// Given a filename preceded by a path, removes the path.  Returns just the filename.
CString StripPath(const CString& filename_with_path)
{
  // Remove path to file if it exists
  int tmpI1 = filename_with_path.ReverseFind('\\');
  int tmpI2 = filename_with_path.ReverseFind('/');

  // If path information exists
  if (!(tmpI1 == -1 && tmpI2 == -1))
  {
    // Set tmpI1 to location of last 'slash'
    if (tmpI1 < tmpI2) tmpI1 = tmpI2;

    // remove path information
    return filename_with_path.Right(filename_with_path.GetLength()-(tmpI1+1));
  }
  else
  {
    return filename_with_path;
  }
}

bool ImageFileFormat::SetLengths(const CString& directory)
{
  CString file_name = GetFirstImage( directory );   // file name of first valid file found
  if (file_name == "")  return false;  // no file found
  
  OrderElement tmpOE;     ///< temp order element
  CString tmpStr;         ///< temp string
  int plen, delim, elen;  ///< length of prefix, location of delim, length of element
  
  // remove path, leaving only filename
  file_name = StripPath( file_name );

  try 
  {
    for (int i = 0; i < NUM_OF_ELEMENTS; i++)
    {
      tmpOE = m_image_file_format_info.InfoOrder[i];
      if(tmpOE.included)
      {
        // find next delimiter
        delim = file_name.Find(m_image_file_format_info.delim);

        // set cut to end if last element
        if(delim == -1)
        { 
          if (i == m_image_file_format_info.numElementsInFileName - 1)  delim = file_name.GetLength();
          else
          {
            assert(0);  // ran out of elements too soon.  Bad image file format.
            return false;
          }
        }

        // get length of prefix
        plen = tmpOE.prefix.GetLength();

        // extract element minus prefix, set element length
        elen = delim - plen;
        tmpStr = file_name.Mid(plen,elen);
        
        // set lengths here!
        if(tmpStr.Right(4) == ".jpg") // 4 is length of ".jpg"
          m_image_file_format_info.InfoOrder[i].length = tmpStr.GetLength() - 4;
        else
          m_image_file_format_info.InfoOrder[i].length = tmpStr.GetLength();
        
        // cut off previous element
        file_name = file_name.Right(file_name.GetLength()-(delim+1));
      } // if element is included
    } // for number of elements in file
  }// try this
  catch (...) 
  {
    assert(0);  // can this even happen?  Who would throw an exception?  -- STB 
    TRACE("Error reading image file.\n\nThe file that caused the error was:\n'"+file_name+"'");
    return false;
  }
  return true;
}

bool ImageFileFormat::ImageFileExistsInDirectory(const CString& directory) const
{
  CFileFind ff;

  return ff.FindFile(directory + GetFileSearchString()) ? true : false;
}

// obsolete
//bool ImageFileFormat::FindNavigationLimits(const CString& directory, int* max_window, int* min_window,
//                                           int* max_session,int* min_session) const
//{
//  // sets element lengths
////  if(!SetLengths(directory)) return false;  // Does not appear necessary -- STB 5/7/07
//
//  CFileFind ff;
//  ImageInfo ii;
//
//  if(ff.FindFile(directory + GetFileSearchString()))
//  {
//    while (1)
//    {
//      bool more = ff.FindNextFile() != 0;
//      ii = GetImageInfo(ff.GetFileName());
//      if(ii.window > *max_window) *max_window = ii.window;
//      if(ii.window < *min_window) *min_window = ii.window;
//      if(ii.session > *max_session) *max_session = ii.session;
//      if(ii.session < *min_session) *min_session = ii.session;
//      if (!more)  break;
//    }
//    return true;
//  }
//  else
//  {
//    return false;  // unable to find an image
//  }
//}

int ImageFileFormat::CountFilesInDirectory(const CString& directory) const
{
  int count = 0;
  
  CFileFind ff;
  if (ff.FindFile(directory + GetFileSearchString()))
  {
    while (1)
    {
      bool more = ff.FindNextFile() != 0;
      // skip over the dot directories . and ..
      if (!ff.IsDots() && !ff.IsDirectory())
      {
        count++;
      }
      if (!more)  break;
    }
  }
  
  return count;
}


// obsolete
// same as above except includes a progress bar prototype
//bool ImageFileFormat::FindNavigationLimits(const CString& directory, int* max_window, int* min_window,
//                                           int* max_session,int* min_session, CWnd *pParentWnd) const
//{
//  // sets element lengths
//  //  if(!SetLengths(directory)) return false;  // Does not appear necessary -- STB 5/7/07
//  
//  CFileFind ff;
//  ImageInfo ii;
//  int lower, upper;
//  CProgressCtrl progress_bar;
//  CRect rect, rect2, rect3;
//  CClientDC dc(pParentWnd);
//  dc.SetBkColor(::GetSysColor(COLOR_3DFACE));
//  pParentWnd->GetWindowRect(rect);
//  pParentWnd->ScreenToClient(rect);
//  rect2 = CRect(rect.left+(rect.Width()/2)-150, rect.top+(rect.Height()/2)-15, rect.left+(rect.Width()/2)+150, rect.top+(rect.Height()/2)+15);
//  rect3 = CRect(rect2.left,rect2.top-20,rect2.right,rect2.bottom-20);
//  dc.DrawText("Finding Images In Directory...         ",rect3,0);
//  progress_bar.Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH , rect2, pParentWnd,0x16);
//  lower = 0;
//  upper = CountFilesInDirectory(directory);
//  progress_bar.SetRange(lower,upper);
//  progress_bar.SetStep(1);
//  
//  
//  if(ff.FindFile(directory + GetFileSearchString()))
//  {
//    while (1)
//    {
//      bool more = ff.FindNextFile() != 0;
//      if(progress_bar.GetPos() == upper) progress_bar.SetPos(lower);
//      progress_bar.StepIt();
//      ii = GetImageInfo(ff.GetFileName());
//      if(ii.window > *max_window) *max_window = ii.window;
//      if(ii.window < *min_window) *min_window = ii.window;
//      if(ii.session > *max_session) *max_session = ii.session;
//      if(ii.session < *min_session) *min_session = ii.session;
//      if (!more)  break;
//    }
//    CBrush background_brush(::GetSysColor(COLOR_3DFACE));
//    dc.FillRect(rect3,&background_brush);
//    progress_bar.ShowWindow(SW_HIDE);
//    return true;
//  }
//  else
//  {
//    CBrush background_brush(::GetSysColor(COLOR_3DFACE));
//    dc.FillRect(rect3,&background_brush);
//    progress_bar.ShowWindow(SW_HIDE);
//    return false;  // unable to find an image
//  }
//}

bool ImageFileFormat::CatalogueImages(const CString& directory, int* max_window, int* min_window,
                                      int* max_session, int* min_session, void *controller) const
{
  // sets element lengths
  //  if(!SetLengths(directory)) return false;  // Does not appear necessary -- STB 5/7/07

  CFileFind ff;
  ImageInfo ii;

  if(ff.FindFile(directory + GetFileSearchString()))
  {
    while (1)
    {
      bool more = ff.FindNextFile() != 0;
      ii = GetImageInfo(ff.GetFileName());
      if(ii.window > *max_window) *max_window = ii.window;
      if(ii.window < *min_window) *min_window = ii.window;
      if(ii.session > *max_session) *max_session = ii.session;
      if(ii.session < *min_session) *min_session = ii.session;

      ((Controller *)controller)->C_SetImageExists(true, ii.tube, ii.window, ii.session);
      if (!more)  break;
    }
    return true;
  }
  else
  {
    return false;  // unable to find an image
  }
}


// iterates over all files in the directory, noting navigation limits and existing files while updating a progress bar
bool ImageFileFormat::CatalogueImages(const CString& directory, int* max_window, int* min_window,
                                           int* max_session,int* min_session, CWnd *pParentWnd, void *controller) const
{  // TODO: this function, along with FindNavigationLimits, directly manipulatee the main window - they probably no longer belong in this file


  // sets element lengths
  //  if(!SetLengths(directory)) return false;  // Does not appear necessary -- STB 5/7/07
  
  CFileFind ff;
  ImageInfo ii;
  
  // progress bar manipulation
  int lower, upper;
  CProgressCtrl progress_bar;
  CRect rect, rect2, rect3;
  CClientDC dc(pParentWnd);
  dc.SetBkColor(::GetSysColor(COLOR_3DFACE));
  pParentWnd->GetWindowRect(rect);
  pParentWnd->ScreenToClient(rect);
  rect2 = CRect(rect.left+(rect.Width()/2)-150, rect.top+(rect.Height()/2)-15, rect.left+(rect.Width()/2)+150, rect.top+(rect.Height()/2)+15);
  rect3 = CRect(rect2.left,rect2.top-20,rect2.right,rect2.bottom-20);
  dc.DrawText("Finding Images In Directory...         ",rect3,0);
  progress_bar.Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH , rect2, pParentWnd,0x16);
  lower = 0;
  upper = CountFilesInDirectory(directory);
  progress_bar.SetRange(lower,upper);
  progress_bar.SetStep(1);
  
  
  if(ff.FindFile(directory + GetFileSearchString()))
  {
    while (1)
    {
      bool more = ff.FindNextFile() != 0;
      if(progress_bar.GetPos() == upper) progress_bar.SetPos(lower);
      progress_bar.StepIt();
      ii = GetImageInfo(ff.GetFileName());
      if(ii.window > *max_window) *max_window = ii.window;
      if(ii.window < *min_window) *min_window = ii.window;
      if(ii.session > *max_session) *max_session = ii.session;
      if(ii.session < *min_session) *min_session = ii.session;

      ((Controller *)controller)->C_SetImageExists(true, ii.tube, ii.window, ii.session);
      if (!more)  break;
    }
    CBrush background_brush(::GetSysColor(COLOR_3DFACE));
    dc.FillRect(rect3,&background_brush);
    progress_bar.ShowWindow(SW_HIDE);
    return true;
  }
  else
  {
    CBrush background_brush(::GetSysColor(COLOR_3DFACE));
    dc.FillRect(rect3,&background_brush);
    progress_bar.ShowWindow(SW_HIDE);
    return false;  // unable to find an image
  }
}

vector<Date> ImageFileFormat::GetDatesFromFile(const CString& dir, int max_sessions, CWnd *pParentWnd)
{
  CString temp_str = "", temp;
  OrderElement tmp;
  CFileFind ff;
  ImageInfo ii;
  Date temp_date;
  int lower, upper;
  lower = 0;
  upper = max_sessions;
  CProgressCtrl progress_bar;
  CRect rect, rect2, rect3;
  CStatic progress_text;
  CClientDC dc(pParentWnd);
  dc.SetBkColor(::GetSysColor(COLOR_3DFACE));
  pParentWnd->GetWindowRect(rect);
  pParentWnd->ScreenToClient(rect);
  rect2 = CRect(rect.left+(rect.Width()/2)-150, rect.top+(rect.Height()/2)-15, rect.left+(rect.Width()/2)+150, rect.top+(rect.Height()/2)+15);
  rect3 = CRect(rect2.left,rect2.top-20,rect2.right,rect2.bottom-20);
  dc.DrawText("Reading Dates From File...         ",rect3,0);
  progress_bar.Create(WS_CHILD | WS_VISIBLE | PBS_SMOOTH , rect2, pParentWnd,0x16);
  progress_bar.SetRange(lower,upper);
  progress_bar.SetStep(1);
  

  // finds all dates in file
  for(int index = 1; index <= max_sessions; index++)
  {
    progress_bar.StepIt();
    // gets the search string
    temp_str = "";
    for(int i = 0; i < NUM_OF_ELEMENTS; i++)
    {
      tmp = m_image_file_format_info.InfoOrder[i];  
      if(tmp.included)
      {
        if(tmp.element == SESSION)
        {
          temp.Format("*%d",index);
          temp_str += tmp.prefix;
          temp_str += temp;
          temp_str += m_image_file_format_info.delim;
        } else
        {
          temp_str += tmp.prefix;
          temp_str += "*";
          temp_str += m_image_file_format_info.delim;
        }
      } // if included
    } // for number of elements in filename
    
    temp_str.SetAt(temp_str.GetLength()-1,'.');
    temp_str += m_image_file_format_info.filetype;
    
    temp_date.month = -1;
    temp_date.day = -1;
    temp_date.year = -1;
    if(ff.FindFile(dir + temp_str))
    {
      ff.FindNextFile();
      temp_str = ff.GetFileName();      
      ii = GetImageInfo(temp_str);
      temp_date = ii.date;
      
    }
    m_dates_from_file.push_back(temp_date);
  }
  CBrush background_brush(::GetSysColor(COLOR_3DFACE));
  dc.FillRect(rect3,&background_brush);
  progress_bar.ShowWindow(SW_HIDE);
  return m_dates_from_file;
}

vector<Date> ImageFileFormat::GetDatesFromFile(const CString& dir, int max_sessions)
{
  CString temp_str = "", temp;
  OrderElement tmp;
  CFileFind ff;
  ImageInfo ii;
  Date temp_date;
  
  // finds all dates in file
  for(int index = 1; index <= max_sessions; index++)
  {
    // gets the search string
    temp_str = "";
    for(int i = 0; i < NUM_OF_ELEMENTS; i++)
    {
      tmp = m_image_file_format_info.InfoOrder[i];  
      if(tmp.included)
      {
        if(tmp.element == SESSION)
        {
          temp.Format("*%d",index);
          temp_str += tmp.prefix;
          temp_str += temp;
          temp_str += m_image_file_format_info.delim;
        } else
        {
          temp_str += tmp.prefix;
          temp_str += "*";
          temp_str += m_image_file_format_info.delim;
        }
      } // if included
    } // for number of elements in filename
    
    temp_str.SetAt(temp_str.GetLength()-1,'.');
    temp_str += m_image_file_format_info.filetype;
    
    temp_date.month = -1;
    temp_date.day = -1;
    temp_date.year = -1;
    if(ff.FindFile(dir + temp_str))
    {
      ff.FindNextFile();
      temp_str = ff.GetFileName();      
      ii = GetImageInfo(temp_str);
      temp_date = ii.date;
      
    }
    m_dates_from_file.push_back(temp_date);
  }

  return m_dates_from_file;
}

ImageInfo ImageFileFormat::GetImageInfo(const CString& filename) const
{

  OrderElement tmpOE;            // temp order element
  CString expt, tube, initials;  // experiment name, tube id, initials
  int window, session;           // window #, session #
  int count = 0, year_size;
  Date date;                     // date

  CString tmpStr;                // tmp string
  CString fn = filename;         // non-const version of filename
  int plen, delim, elen;         // prefix length, location of delim, element length
  Date_Format d_format;          // temp varible used for date format

  d_format = m_image_file_format_info.m_date_format;
  
  // Remove path to file if it exists
  int tmpI1 = fn.ReverseFind('\\');
  int tmpI2 = fn.ReverseFind('/');

  // If path information exists
  if(!(tmpI1 == -1 && tmpI2 == -1))
  {
    // Set tmpI1 to location of last 'slash'
    if(tmpI1 < tmpI2) tmpI1 = tmpI2;

    // remove path information
    fn = filename.Right(filename.GetLength()-(tmpI1+1));
  }

  try {
    for(int i = 0; i < NUM_OF_ELEMENTS; i++)
    {
      tmpOE = m_image_file_format_info.InfoOrder[i];
      if(tmpOE.included)
      {
        // find next delimiter
        delim = fn.Find(m_image_file_format_info.delim);

        // set cut to end if last element
        if(delim == -1)
        { 
          if(i == m_image_file_format_info.numElementsInFileName-1) delim = fn.GetLength();
          else TRACE("Error in GetImageInfo - ran out of elements too soon.");
        }

        // get length of prefix
        plen = tmpOE.prefix.GetLength();

        // extract element minus prefix, set element length
        elen = delim-plen;
        tmpStr = fn.Mid(plen,elen);

        // set appropriate element
        int i;
        switch(tmpOE.element)
        {
          case EXPT: expt = tmpStr; break;
          case TUBE: tube = tmpStr; break;
          case WINDOW: window = atoi(tmpStr); break;
          case SESSION: session = atoi(tmpStr); break;
          case SDATE:
                  
                  // gets the date from file in the specified order
                  for(i = 0; i < d_format.order.size(); i++)
                  {
                    if(d_format.order.at(i) == MONTH)
                    {
                      date.month = atoi(tmpStr.Mid(count,2));
                      count += 3;                             // includes separator
                    } else if(d_format.order.at(i) == DAY)
                    {
                      date.day = atoi(tmpStr.Mid(count,2));
                      count += 3;                             // includes separator
                    } else if(d_format.order.at(i) == YEAR)
                    {
                      
                      if(d_format.four_year) year_size = 4;
                      else year_size = 2;
                      date.year = atoi(tmpStr.Mid(count,year_size));
                      count += year_size + 1;                             // includes separator
                    }
                  }
                  /* yyyy.mm.dd
                  date.year = atoi(tmpStr.Left(4));
                  date.month = atoi(tmpStr.Mid(5,2));
                  date.day = atoi(tmpStr.Right(2)); */
                  break;
          case INITIALS: initials = tmpStr; break;
        }
        
        // cut off previous element
        fn = fn.Right(fn.GetLength()-(delim+1));

      } // if element is included
    } // for number of elements in file
  }// try this
  catch (...) {
    TRACE("Error reading image file.\n\nThe file that caused the error was:\n'"+filename+"'");
  }

  // Create ImageInfo instance and return
  ImageInfo ii;
  ii.expt = expt;
  ii.tube = tube;
  ii.window = window;
  ii.session = session;
  ii.date = date;
  ii.initials = initials;

  return ii;
}

ImageInfo ImageFileFormat::GetImageInfoOfFirstFile(const CString& directory) const
{
//  SetLengths(directory);  // Does not appear necessary -- STB 5/7/07

  CFileFind ff;
  if(ff.FindFile(directory + GetFileSearchString()))
  {
    ff.FindNextFile();
    return GetImageInfo(directory + ff.GetFileName());
  }
  else return ImageInfo();  // if file not found, return empty struct
}

CString ImageFileFormat::GetFirstImage(const CString& directory) const
{
  CFileFind ff;
  if(ff.FindFile(directory + GetFileSearchString()))
  {
    ff.FindNextFile();
    return directory + ff.GetFileName();
  }
  else return "";
}

CRect ImageFileFormat::GetImageRectangle(const CString& filename)
{
  CRect rect(0,0,0,0);

  blepo::ImgBgr img;

  bool success = blepo::Load(filename,&img);

  if(success)
  {
    rect.right = img.Width();
    rect.bottom = img.Height();
  }
  assert(rect.right < 2000);
  return rect;
}

void ImageFileFormat::SetImageFileFormatInfo(const ImageFileFormatInfo& iffinfo)
{
  m_image_file_format_info = iffinfo;
}

const ImageFileFormatInfo& ImageFileFormat::GetImageFileFormatInfo() const
{
  return m_image_file_format_info;
}

bool ImageFileFormat::DateIsIncluded() const
{
  bool date_is_included = false;

  // must look through elements 1 by 1 until date element is found
  for(int i = 0; i < NUM_OF_ELEMENTS; i++)
  {
    if(m_image_file_format_info.InfoOrder[i].element == SDATE)
    { date_is_included = m_image_file_format_info.InfoOrder[i].included; break; }
  }

  return date_is_included;
}

void ImageFileFormat::SetToBartz()
{
  InitializeDateFormat();
  m_image_file_format_info.delim = '_';
  m_image_file_format_info.filetype = "jpg";
  m_image_file_format_info.numElementsInFileName = 7;

  vector<OrderElement> v;
  OrderElement x;

  x.element = EXPT;
  x.prefix = "";
  x.length = 0;
  x.included = TRUE;
  v.push_back(x);

  x.element = TUBE;
  x.prefix = "T";
  x.length = 3;
  v.push_back(x);

  x.element = WINDOW;
  x.prefix = "L";
  v.push_back(x);
  
  x.element = SDATE;
  x.prefix = "";
  x.length = 0;
  v.push_back(x);
  
  x.element = TIME;
  v.push_back(x);
  
  x.element = SESSION;
  x.length = 3;
  v.push_back(x);
  
  x.element = INITIALS;
  x.length = 0;
  v.push_back(x);

  x.element = MISC1;
  x.included = FALSE;
  v.push_back(x);

  x.element = MISC2;
  v.push_back(x);

  x.element = MISC3;
  v.push_back(x);

  x.element = MISC4;
  v.push_back(x);
  
  m_image_file_format_info.InfoOrder = v;
}

void ImageFileFormat::InitializeDateFormat()
{  
  // initialize date format       yyyy.mm.dd
  m_image_file_format_info.m_date_format.date_delim = ".";
  m_image_file_format_info.m_date_format.four_year = true;
  m_image_file_format_info.m_date_format.order.clear();
  m_image_file_format_info.m_date_format.order.push_back(YEAR);  
  m_image_file_format_info.m_date_format.order.push_back(MONTH);
  m_image_file_format_info.m_date_format.order.push_back(DAY);
  
}

void ImageFileFormat::Save(FILE* fp) const
{
  g_debug_logfile.Print("ImageFileFormat::Save() 1\n");
  // Write image format string
  char image_format[1024];
  g_debug_logfile.Print("ImageFileFormat::Save() 2\n");
  sprintf(image_format,"%d %s %c",m_image_file_format_info.numElementsInFileName,
                                  (const char*) m_image_file_format_info.filetype,
                                  m_image_file_format_info.delim);
  g_debug_logfile.Print("ImageFileFormat::Save() 3 %s\n", image_format);

  OrderElement oe;
  g_debug_logfile.Print("ImageFileFormat::Save() 4 %d\n", NUM_OF_ELEMENTS);
  for(int i = 0; i < NUM_OF_ELEMENTS; i++)
  {
  g_debug_logfile.Print("ImageFileFormat::Save() 4b %d\n", i);
	  oe = m_image_file_format_info.InfoOrder[i];
  g_debug_logfile.Print("ImageFileFormat::Save() 4c %s %s\n", image_format, oe.prefix);
    sprintf(image_format,"%s %d %s %d %d",image_format,oe.element,oe.prefix,oe.length,oe.included);
  g_debug_logfile.Print("ImageFileFormat::Save() 4d\n");
  }
  g_debug_logfile.Print("ImageFileFormat::Save() 5\n");

  fprintf(fp,"%s\n",image_format);
  g_debug_logfile.Print("ImageFileFormat::Save() 6\n");

  // saves date format
  // order-{elem1 elem2 elem3} four_year separator
  g_debug_logfile.Print("ImageFileFormat::Save() 7\n");
  Date_Format d_format = m_image_file_format_info.m_date_format;
  fprintf(fp,"%d %d %d %d %s\n", d_format.order.at(0), d_format.order.at(1), d_format.order.at(2),
                                 d_format.four_year, d_format.date_delim);
  g_debug_logfile.Print("ImageFileFormat::Save() 8\n");
}

char* ImageFileFormat::Load(FILE* fp)
{
  char buffer[1024];

  //Read the image format with spaces
  char* status = fgets (buffer, 1024, fp);
  if(status != NULL)
  {
    buffer[strlen(buffer)-1] = '\0';
    string image_format = buffer;
    
    ImageFileFormatInfo iffi;
    OrderElement oe;
    string::iterator it = image_format.begin();
    vector<string> elems;
    string tmp = "";
    
    // Get elements one character at a time
    // \0 is end of data
    // ' ' is end of element
    while(*it != '\0')
    {
      if(*it != ' ')
      {
        tmp.append(1,*it);
      }
      else
      {
        elems.push_back(tmp);
        tmp = "";
      }
      it++;
    }
    elems.push_back(tmp);
    
    // store numElementsInFileName, filetype, and delimiter
    iffi.numElementsInFileName = atoi(elems[0].c_str());
    iffi.filetype = elems[1].c_str();
    iffi.delim = elems[2][0];
    
    // store name, element, prefix, length, and included for each Element(expt, tube, etc)
    // 3 is next string element in image_format (0,1,2 are used above)
    // 4 is number of data elements in OrderElement structure
    for(int i = 3; i < 3 + NUM_OF_ELEMENTS*4; i+=4)
    {
      oe.element = (ELEMENT)atoi(elems[i].c_str());
      oe.prefix = elems[i+1].c_str();
      oe.length = atoi(elems[i+2].c_str());
      oe.included = atoi(elems[i+3].c_str()) > 0 ? true : false;
      iffi.InfoOrder.push_back(oe);
    }
    
    m_image_file_format_info = iffi;
  }

  // loads the date format
  int temp1,temp2,temp3;
  bool temp4;
  Date_Format d_format;
  char temp5[1000];
  fscanf(fp,"%d %d %d %d %s\n", &temp1, &temp2, &temp3, &temp4, &temp5);
  d_format.date_delim = temp5;
  d_format.order.push_back(temp1);
  d_format.order.push_back(temp2);
  d_format.order.push_back(temp3);
  d_format.four_year = temp4;
  m_image_file_format_info.m_date_format = d_format;

  return status;
}